@@ -1,34 +1,24 @@
 /*
-  This file is part of the KDE project
+    This file is part of the KDE project
 
-  Copyright (c) 2007 Andreas Hartmetz <ahartmetz@gmail.com>
-  Copyright (c) 2007 Michael Jansen <kde@michael-jansen.biz>
+    SPDX-FileCopyrightText: 2007 Andreas Hartmetz <ahartmetz@gmail.com>
+    SPDX-FileCopyrightText: 2007 Michael Jansen <kde@michael-jansen.biz>
 
-  This library is free software; you can redistribute it and/or
-  modify it under the terms of the GNU Library General Public
-  License version 2 as published by the Free Software Foundation.
-
-  This library is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-  Library General Public License for more details.
-
-  You should have received a copy of the GNU Library General Public License
-  along with this library; see the file COPYING.LIB.  If not, write to
-  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-  Boston, MA 02110-1301, USA.
-  */
+    SPDX-License-Identifier: LGPL-2.0-only
+*/
 
 #include "kglobalacceld.h"
 #include "logging_p.h"
 
-#include <kcrash.h>
 #include <KAboutData>
+#include <KCrash>
 #include <KDBusService>
-#include <QDebug>
-#include <QGuiApplication>
-#include <QIcon>
 #include <QCommandLineParser>
+#include <QGuiApplication>
+
+#ifdef Q_OS_UNIX
+#include <unistd.h>
+#endif
 
 static bool isEnabled()
 {
@@ -44,16 +34,15 @@ extern "C" Q_DECL_EXPORT int main(int argc, char **argv)
     // registering the app with the session manager. We remove the address to make sure we do not
     // get a hang on kglobalaccel restart (kglobalaccel tries to register with ksmserver,
     // ksmserver tries to register with kglobalaccel).
-    qunsetenv( "SESSION_MANAGER" );
+    qunsetenv("SESSION_MANAGER");
 
     QGuiApplication app(argc, argv);
-    KAboutData aboutdata(
-            QStringLiteral("kglobalaccel"),
-            QObject::tr("KDE Global Shortcuts Service"),
-            QStringLiteral("0.2"),
-            QObject::tr("KDE Global Shortcuts Service"),
-            KAboutLicense::LGPL,
-            "(C) 2007-2009  Andreas Hartmetz, Michael Jansen");
+    KAboutData aboutdata(QStringLiteral("kglobalaccel"),
+                         QObject::tr("KDE Global Shortcuts Service"),
+                         QStringLiteral("0.2"),
+                         QObject::tr("KDE Global Shortcuts Service"),
+                         KAboutLicense::LGPL,
+                         "(C) 2007-2009  Andreas Hartmetz, Michael Jansen");
     aboutdata.addAuthor("Andreas Hartmetz", QObject::tr("Maintainer"), "ahartmetz@gmail.com");
     aboutdata.addAuthor("Michael Jansen", QObject::tr("Maintainer"), "kde@michael-jansen.biz");
 
@@ -72,9 +61,21 @@ extern "C" Q_DECL_EXPORT int main(int argc, char **argv)
         return 0;
     }
 
+#ifdef Q_OS_UNIX
+    // It's possible that kglobalaccel gets started as the wrong user by
+    // accident, e.g. kdesu dolphin leads to dbus activation. It then installs
+    // its grabs and the actions are run as the wrong user.
+    bool isUidset = false;
+    const int sessionuid = qEnvironmentVariableIntValue("KDE_SESSION_UID", &isUidset);
+    if(isUidset && static_cast<uid_t>(sessionuid) != getuid()) {
+        qCWarning(KGLOBALACCELD) << "kglobalaccel running as wrong user, exiting.";
+        return 0;
+    }
+#endif
+
     KDBusService service(KDBusService::Unique);
 
-    app.setQuitOnLastWindowClosed( false );
+    app.setQuitOnLastWindowClosed(false);
     QGuiApplication::setFallbackSessionManagementEnabled(false);
 
     // Restart on a crash
